import terrylib.*;

import terrylib.util.*;
import openfl.display.*;
import openfl.geom.*;
import openfl.events.*;
import openfl.net.*;
import openfl.text.*;
import openfl.Assets;
import openfl.Lib;
import openfl.system.Capabilities;


class Main {

	var hairs:Array<BitmapData>;

	var hairLength=70;

	function rndFloat(min:Float,max:Float):Float {
		return min+Math.random()*(max-min);
	}
	
	function rndInt(min:Int,max:Int):Int {
		return min+Math.floor(Math.random()*(max-min));
	}
	
	var scragginess:Float=60*Math.PI/180.0;
		
	var fallCount=0;
	var marginX=20;
	var marginY=20;
	var dx=10;
	var dy=10;
	var xCount=50;
	var yCount=30;

	var patchX:Int;
	var patchY:Int;
	var patchSize:Int=40;

	var fallChance=0.03;

	var angles:Array<Float>;
	var falling:Array<Float>;

	var phase=0;

	var ox=-1;
	var oy=-1;

	var radius=30;
	var sensitivity=5;

	function new() {
		patchX=marginX+(xCount)*dx-patchSize*2;
		patchY=marginY+(yCount)*dy-patchSize*2;
		trace((marginX+patchSize*2)+","+ (marginY+patchSize*2)+" < "+patchX+","+patchY+" < " + (marginX+(xCount-1)*dx-patchSize*2)+ "," + (marginY+(yCount-1)*dy-patchSize*2));

		ox=Mouse.x;
		oy=Mouse.y;


		angles = new Array<Float>();
		falling = new Array<Float>();
		for (x in 0...xCount){
			for (y in 0...yCount){
				angles.push(rndFloat(0,360));
				falling.push(-1);
			}
		}

		hairs = new Array<BitmapData>();
		for (i in 0...360) {
			var bmd : BitmapData = new BitmapData(hairLength*2+1,hairLength*2+1,true,0);
			
			var x:Float=hairLength;
			var y:Float=hairLength;
			var a:Float=i*Math.PI/180.0;
			bmd.lock();
			
			for (j in 0...hairLength){
				if (j%5==0){
					a = i*Math.PI/180.0+rndFloat(-scragginess,scragginess);
				}
				bmd.setPixel32(Math.floor(x),Math.floor(y),0xffee7733);
				x+=Math.cos(a);
				y+=Math.sin(a);
			}

			bmd.unlock();
			hairs.push(bmd);
		}	
	}
	
	function dist(x1:Int,y1:Int,x2:Int,y2:Int){
		var d_x=x2-x1;
		var d_y=y2-y1;
		return Math.sqrt(d_x*d_x+d_y*d_y);
	}

	var lerpSpeed=1.0;

	function lerp(a:Float,b:Float,t:Float):Float{
		return a+ (b-a)*t;
	}

	function repeat( t:Float,  length:Float):Float
	{
		return t - Math.floor(t / length) * length;
	}

	function lerpAngle( a:Float,  b:Float,  t:Float ):Float
	{
		var num = repeat(b - a, 360);
		if (num > 180)
		{
			num -= 360;
		}

		if (t<0)t=0;
		if (t>1)t=1;

		return a + num * t;
	}


	function hairComb(){
		if(ox==Mouse.x && oy==Mouse.y) {
			return;
		}
		
		var diffx = Mouse.x-ox;
		var diffy = Mouse.y-oy;
		var l = dist(Mouse.x,Mouse.y,ox,oy);

		for (x in 0...xCount){
			for (y in 0...yCount){
				if (falling[x+xCount*y]>=0){
					continue;
				}
				var px = marginX+x*dx+hairLength;
				var py = marginY+y*dy+hairLength;
				var d=Math.min(dist(Mouse.x,Mouse.y,px,py),dist(ox,oy,px,py));
				if (d<radius){
					var a = Math.atan2(diffy,diffx)*180/Math.PI;
					var fromA=angles[x+xCount*y] ;
					a=lerpAngle(a,fromA,(d-sensitivity)/(radius-sensitivity));
					a = ((a%360)+360)%360;
					angles[x+xCount*y] = a;	
					if (Math.random()<=fallChance){
						if (dist(Mouse.x,Mouse.y,patchX,patchY)<=(patchSize)){
							falling[x+xCount*y]=0;
							fallCount++;
						}
					}
				}
			}
		}
		ox=Mouse.x;
		oy=Mouse.y;
	}

	function update() {
		phase=(phase+1)%360;

		for (x in 0...xCount){
			for (y in 0...yCount){			
				var a:Float = angles[x+xCount*y];
				var bmd:BitmapData = hairs[Math.floor(a)];
				var px:Int = marginX+dx*x;
				var py:Int = marginY+dy*y;
				if (falling[x+xCount*y]>=0){
					falling[x+xCount*y]++;
					Gfx.drawimage_bmd(px,py+Math.floor(falling[x+xCount*y]),bmd);
				} else {
					Gfx.drawimage_bmd(px,py,bmd);
				}
			}
		}

		hairComb();
	}
}